import { shallowEqual } from "react-redux"
import { Button, TextField } from '@mui/material';
import Box from '@mui/material/Box';
import Modal from '@mui/material/Modal';
import { AppDispatch, AppState, useAppDispatch } from "../../store/store"
import { useSelector } from "react-redux"
import Preloader from "../../components/UI/Preloader/Preloader"
import { ChangeEvent, FormEvent, useEffect, useState } from "react"
import IPhotoDto from "../../interfaces/IPhotoDto"
import { addPhoto } from "../../store/photos/photos.slice"
import styles from './AddForm.module.css'

const AddForm: React.FunctionComponent = (): React.ReactElement => {

    const {loadingPhotos} = useSelector((state: AppState) => state.photos, shallowEqual)

    const [buttonDisabled, setButtonDisabled] = useState<boolean>(true) 

    const [fileName, setFileName] = useState<string>('')

    const [photoDto, setPhotoDto] = useState<IPhotoDto>({
        title: '',
        photo: undefined
    })

    const dispatch: AppDispatch = useAppDispatch()

    const inputHandler = (e: ChangeEvent<HTMLInputElement>): void => {
        setPhotoDto(prevState => {
            return {...prevState, [e.target.name]: e.target.value}
        })
    }

    const inputFileHandler = (e: ChangeEvent<HTMLInputElement>): void => {
        setPhotoDto(prevState => {
            return {...prevState, 
                photo: e.target.files ? e.target.files[0] : undefined}
        })
        setFileName(e.target.files && e.target.files[0] ? e.target.files[0].name : '')
    }

    const checkButton = () => {
        if(photoDto.title.trim() === '' || !photoDto.photo){
            setButtonDisabled(true)
        } else{
            setButtonDisabled(false)
        }
    }

    const submitHandler = (e: FormEvent): void => {
        e.preventDefault()
        const formData = new FormData()
        Object.keys(photoDto).forEach((key: string) => {
            //@ts-ignore
            formData.append(key, photoDto[key])
        })
        dispatch(addPhoto(formData))
        setPhotoDto({
            title: '',
            photo: undefined
        })
        setFileName('')
    }

    useEffect(() => {
        checkButton()
    },[photoDto])
    
    return(
        <div
        >
            {
                loadingPhotos ? <Preloader/> : null
            }
            <h1>Add new photo</h1>
            <form
                onSubmit={submitHandler}
                style={{margin: '10px', padding: '20px', background: '#e0e0e0', borderRadius: '5px', display: 'flex', flexDirection: 'column'}}
            >
            <TextField 
                id="outlined-basic" 
                label="Photo title" 
                variant="outlined" 
                style={{marginBottom: '20px'}}
                value={photoDto.title}
                name='title'
                onChange={inputHandler}
                autoComplete='off'
            />
            <label style={{height: 'auto', margin: '10px', maxWidth: '30%', display: 'flex', flexDirection: 'column'}}>
                <input
                    style={{display: "none"}}
                    name={'photo'}
                    type="file"
                    placeholder="Image"
                    accept=".png, .jpg, .jpeg"
                    onChange={inputFileHandler}
                />
                <h1 className={styles.file_input} style={{margin: '0'}}>
                    Choose file
                </h1>
                <span className={styles.filename}>{fileName}</span>
            </label>
            <button 
                disabled={buttonDisabled}   
            className={styles.add_btn}>SEND</button>
            </form>
        </div>
    )
}

export default AddForm